#coding: utf-8

import copy
from lxml import etree


def fill_object_with_data(obj, data, mapping):
    u'''
    Заполнение аттрибутов объекта из внешнего словаря
    на основе существующего отображения.

    Отображение должен иметь следующий формат:
    {
        ключ_в_словаре: аттрибут_в_объекте
    }
    или
    {
        ключ_в_словаре: (аттрибут_в_объекте, обработчик)
    }

    Если по ключу лежит tuple, то в объект будет помещено
    значение, пропущенное предварительно через обработчик.
    Обработчик должен принимать один аргумент и возвращать
    одно значение. Самый простой пример: (lambda x: x)
    '''
    for key, way in mapping.iteritems():
        if not key in data:
            continue

        value = data[key]
        if isinstance(way, tuple):
            (attr_name, converter) = way
            value = converter(value)
        else:
            attr_name = way
        setattr(obj, attr_name, value)


def iterate_obj_file(fp, value_tag):
    u'''
    Итеративная обработка XML-файла с объектами. Поскольку все
    записи в ФИАС имеют форму замкнутого тэга со значимой информацией
    в аттрибутах, мы можем просто отлавливать событие окончания парсинга
    этого тэга и возвращать его копию в виде словаря.

    Так как при итеративной обработке все равно строится дерево тэгов,
    для экономии памяти мы удаляем содержимое тэга и всех его родителей.
    '''

    feed = etree.iterparse(fp, events=('end',))
    for action, elem in feed:
        result = copy.copy(elem.attrib)
        elem.clear()
        while elem.getprevious() is not None:
            del elem.getparent()[0]

        if elem.tag == value_tag:
            yield result
