#coding: utf-8

from datetime import datetime

from fias.models import AddrObject, House, NormativeDocument

parse_date = (lambda s: datetime.strptime(s, '%Y-%m-%d'))
parse_bool = (lambda s: bool(int(s)))

def guid_ref(model, field):
    def loader(value):
        obj = None
        try:
            obj = model.objects.get(**{field: value})
        except model.DoesNotExist:
            obj = None
        return obj
    return loader


def actual_ao_guid_ref(field):
    def loader(value):
        obj = AddrObject.objects.get(**{field: value, 'actual_status': True})
        return obj
    return loader


HOUSE_RECORD_MAPPING = {
    'HOUSEID': 'rec_guid',
    'HOUSEGUID': 'house_guid',
    'AOGUID': 'parent_guid',
    'STARTDATE': ('start_date', parse_date),
    'ENDDATE': ('end_date', parse_date),
    'STATSTATUS': ('stat_status', int),
    #'NORMDOC': ('norm_doc', guid_ref(NormativeDocument, 'normdoc_guid')),
    'POSTALCODE': 'postal_code',
    'UPDATEDATE': ('update_date', parse_date),
    'OKATO': 'okato',
    'OKTMO': 'oktmo',
    'IFNSFL': 'ifnsfl',
    'IFNSUL': 'ifnsul',
    'TERRIFNSFL': 'terr_ifnsfl',
    'TERRIFNSUL': 'terr_ifnsul',
    'HOUSENUM': 'house_number',
    'BUILDNUM': 'building_number',
    'STRUCNUM': 'structure_number',
    'ESTSTATUS': ('estate_status', int)
}


AO_RECORD_MAPPING = {
    'AOID': 'rec_guid',
    'AOGUID': 'ao_guid',
    'PREVID': 'prev_guid',
    'NEXTID': 'next_guid',
    'PARENTGUID': 'parent_guid',
    'FORMALNAME': 'formal_name',
    'OFFNAME': 'official_name',
    'POSTALCODE': 'postal_code',
    'IFNSFL': 'ifnfsl',
    'TERRIFNSFL': 'terr_ifnsfl',
    'IFNSUL': 'ifnsul',
    'TERRIFNSUL': 'terr_ifnsul',
    'OKATO': 'okato',
    'OKTMO': 'oktmo',
    'UPDATEDATE': 'update_date',
    'SHORTNAME': 'shortname',
    'AOLEVEL': ('ao_level', int),
    'CODE': 'kladr_code',
    'ACTSTATUS': ('actual_status', parse_bool),
    'OPERSTATUS': 'oper_status',
    'STARTDATE': ('start_date', parse_date),
    'ENDDATE': ('end_date', parse_date),
    'LIVESTATUS': ('live_status', parse_bool),
    'REGIONCODE': ('region_code', int),
    'AUTOCODE': ('autonomy_code', int),
    'AREACODE': ('area_code', int),
    'CITYCODE': ('city_code', int),
    'CTARCODE': ('ctar_code', int),
    'PLACECODE': ('place_code', int),
    'STREETCODE': ('street_code', int),
    'EXTRCODE': ('extr_code', int),
    'SEXTCODE': ('sext_code', int)
}


NORMDOC_MAPPING = {
    'NORMDOCID': 'normdoc_guid',
    'DOCNAME': 'name',
    'DOCDATE': 'date',
    'DOCNUM': 'number',
    'DOCTYPE': 'doc_type'
}


INTERVAL_RECORD_MAPPING = {
    'AOGUID': ('parent', actual_ao_guid_ref('ao_guid')),
    'HOUSEINTID': 'rec_guid',
    'INTGUID': 'interval_guid',
    'POSTALCODE': 'postal_code',
    'IFNSFL': 'ifnfsl',
    'TERRIFNSFL': 'terr_ifnsfl',
    'IFNSUL': 'ifnsul',
    'TERRIFNSUL': 'terr_ifnsul',
    'OKATO': 'okato',
    'OKTMO': 'oktmo',
    'UPDATEDATE': 'update_date',
    'STARTDATE': ('start_date', parse_date),
    'ENDDATE': ('end_date', parse_date),
    'INTSTATUS': ('interval_status', int),
    'INTSTART': ('start', int),
    'INTEND': ('end', int)
}
