#coding: utf-8

from django.db import models


INTERVAL_STATUS_CHOICES = (
    (0, u'Не определено'),
    (1, u'Обычный'),
    (2, u'Четный'),
    (3, u'Нечетный')
)


ESTATE_STATUS_CHOICES = (
    (0, u'Не определено'),
    (1, u'Владение'),
    (2, u'Дом'),
    (3, u'Домовладение'),
    (4, u'Участок')
)


OPER_STATUS_CHOICES = (
    (1, u'Инициация'),
    (10, u'Добавление'),
    (20, u'Изменение'),
    (21, u'Групповое изменение'),
    (30, u'Удаление'),
    (31, u'Удаление вследствие удаления вышестоящего объекта'),
    (40, u'Присоединение адресного объекта (слияние)'),
    (41, u'Переподчинение вследствие слияния вышестоящего объекта'),
    (42, u'Прекращение существования (присоединение к другому АО)'),
    (43, u'Создание нового АО в результате слияния АО'),
    (50, u'Переподчинение'),
    (51, u'Переподчинение вследствие переподчинения вышестоящего объекта'),
    (60, u'Прекращение существования вследствие дробления'),
    (61, u'Создание нового АО в результате дробления'),
    (70, u'Восстановление прекратившего существование объекта')
)


REGION = 1
BOROUGH = 2
DISTRICT = 3
CITY = 4
INTRACITY = 5
PLACE = 6
STREET = 7
ADD_TERRITORY = 90
SUB_ADD_TERRITORY = 91
ALL_LEVELS = [REGION, BOROUGH, DISTRICT, CITY, INTRACITY, 
              PLACE, STREET, ADD_TERRITORY, SUB_ADD_TERRITORY]


AO_LEVELS_CHOICES = (
    (REGION, u'Регион'),
    (BOROUGH, u'Автономный округ'),
    (DISTRICT, u'Район'),
    (CITY, u'Город'),
    (INTRACITY, u'Внутригородская территория'),
    (PLACE, u'Населенный пункт'),
    (STREET, u'Улица'),
    (ADD_TERRITORY, u'Дополнительная территория'),
    (SUB_ADD_TERRITORY, u'Подчиненный доп. территории объект')
)


class ActualAOManager(models.Manager):
    def get_query_set(self):
        return super(ActualAOManager, self).get_query_set()\
                                           .filter(actual_status=True)


class AddrObject(models.Model):
    rec_guid = models.CharField(
        max_length=36,
        verbose_name=u'GUID записи')
    ao_guid = models.CharField(
        max_length=36,
        verbose_name=u'GUID адресного объекта')
    parent_guid = models.CharField(
        max_length=36,
        verbose_name=u'GUID родительского объекта')
    parent = models.ForeignKey('AddrObject',
        verbose_name=u'Ссылка на родительскую запись',
        null=True)
    prev_guid = models.CharField(
        max_length=36,
        verbose_name=u'Идентификатор предыдущей исторической записи')
    next_guid = models.CharField(
        max_length=36,
        verbose_name=u'Идентификатор следующей исторической записи')

    formal_name = models.CharField(
        max_length=120,
        verbose_name=u'Формализованное наименование')
    official_name = models.CharField(
        max_length=120,
        verbose_name=u'Официальное наименование')
    postal_code = models.CharField(
        max_length=6,
        verbose_name=u'Почтовый индекс')
    ifnsfl = models.CharField(
        max_length=4,
        verbose_name=u'Код ИФНС ФЛ')
    terr_ifnsfl = models.CharField(
        max_length=4,
        verbose_name=u'Код территориального участка ИФНС ФЛ')
    ifnsul = models.CharField(
        max_length=4,
        verbose_name=u'Код ИФНС ЮЛ')
    terr_ifnsul = models.CharField(
        max_length=4,
        verbose_name=u'Код территориального участка ИФНС ЮЛ')
    okato = models.CharField(
        max_length=11,
        verbose_name=u'ОКАТО')
    oktmo = models.CharField(
        max_length=8,
        verbose_name=u'ОКТМО')
    update_date = models.DateField(
        verbose_name='Дата внесения (обновления) записи')
    shortname = models.CharField(
        max_length=10,
        verbose_name=u'Краткое наименование типа объекта')
    ao_level = models.PositiveSmallIntegerField(
        verbose_name=u'Уровень адресного объекта',
        choices=AO_LEVELS_CHOICES)
    kladr_code = models.CharField(
        max_length=17,
        verbose_name=u'Код адресного объекта из КЛАДР 4.0')
    region_code = models.PositiveSmallIntegerField(
        verbose_name=u'Код региона',
        blank=False,
        default=0)
    autonomy_code = models.PositiveSmallIntegerField(
        verbose_name=u'Код автономии',
        blank=False,
        default=0)
    area_code = models.PositiveSmallIntegerField(
        verbose_name=u'Код района',
        blank=False,
        default=0)
    city_code = models.PositiveSmallIntegerField(
        verbose_name=u'Код города',
        blank=False,
        default=0)
    ctar_code = models.PositiveSmallIntegerField(
        verbose_name=u'Код внутригородского района',
        blank=False,
        default=0)
    place_code = models.PositiveSmallIntegerField(
        verbose_name=u'Код населенного пункта',
        blank=False,
        default=0)
    street_code = models.PositiveSmallIntegerField(
        verbose_name=u'Код улицы',
        blank=False,
        default=0)
    extr_code = models.PositiveSmallIntegerField(
        verbose_name=u'Код доп. адресообразующего элемента',
        blank=False,
        default=0)
    sext_code = models.PositiveSmallIntegerField(
        verbose_name=u'Код подчиненного доп. адресообразующего элемента',
        blank=False,
        default=0)
    actual_status = models.BooleanField(
        verbose_name=u'Статус актуальности АО ФИАС')
    oper_status = models.PositiveSmallIntegerField(
        verbose_name=u'Статус действия над записью',
        choices=OPER_STATUS_CHOICES)
    start_date = models.DateField(
        verbose_name=u'Начало действия записи')
    end_date = models.DateField(
        verbose_name=u'Окончание действия записи')
    live_status = models.BooleanField(
        verbose_name=u'Признак действующего АО')

    actual = ActualAOManager()
    objects = models.Manager()

    def address_text(self, with_postal=False, levels=None):
        u'''
        Построение полного адреса с учетом вышестоящих АО.
        '''
        postal_code = self.postal_code
        parts = []    

        # Формат вывода адреса по умолчанию: "г. <город>, р-н <район>, ул. <улица>"    
        if not levels:            
            levels = [CITY, PLACE, STREET]

        node = self.parent
        while node:
            if node.ao_level in levels:
                addr = u'{0}. {1}'.format(node.shortname, node.formal_name)
                parts.insert(0, addr)
            # Если явно не указан почтовый индекс, то берем от вышестоящего АО
            if not postal_code and node.postal_code:
                postal_code = node.postal_code           
            node = node.parent

        if self.ao_level in levels:
            # Выводим почтовый индекс только при явном запросе
            if with_postal and postal_code:
                parts.insert(0, postal_code)
            parts.append(u'{0}. {1}'.format(self.shortname, self.formal_name))
        return u', '.join(parts)

    def __unicode__(self):
        return self.address_text(with_postal=True, levels=ALL_LEVELS)

    class Meta:
        verbose_name = u'Адресный объект ФИАС'
        verbose_name_plural = u'Адресные объекты ФИАС'


class NormativeDocument(models.Model):
    normdoc_guid = models.CharField(
        max_length=36,
        verbose_name=u'GUID нормативного документа',
        unique=True,
        blank=False)
    name = models.TextField(
        verbose_name=u'Наименование документа')
    date = models.DateField(
        verbose_name='Дата документа',
        null=True,
        blank=True)
    number = models.CharField(
        max_length=20,
        verbose_name=u'Номер документа')
    doc_type = models.PositiveIntegerField(
        verbose_name=u'Тип документа',
        default=0)


class House(models.Model):
    rec_guid = models.CharField(
        max_length=36,
        verbose_name=u'GUID записи',
        unique=True,
        blank=False)
    house_guid = models.CharField(
        max_length=36,
        verbose_name=u'GUID дома',
        blank=False)

    parent_guid = models.CharField(
        max_length=36,
        verbose_name=u'GUID дома')
    parent = models.ForeignKey(AddrObject,
        verbose_name=u'Ссылка на родительскую запись',
        null=True)

    start_date = models.DateField(
        verbose_name=u'Начало действия записи',
        blank=False)
    end_date = models.DateField(
        verbose_name=u'Окончание действия записи',
        blank=False)
    stat_status = models.PositiveIntegerField(
        verbose_name=u'Состояние дома',
        blank=False)

    postal_code = models.CharField(
        max_length=6,
        verbose_name=u'Почтовый индекс')
    ifnsfl = models.CharField(
        max_length=4,
        verbose_name=u'Код ИФНС ФЛ')
    terr_ifnsfl = models.CharField(
        max_length=4,
        verbose_name=u'Код территориального участка ИФНС ФЛ')
    ifnsul = models.CharField(
        max_length=4,
        verbose_name=u'Код ИФНС ЮЛ')
    terr_ifnsul = models.CharField(
        max_length=4,
        verbose_name=u'Код территориального участка ИФНС ЮЛ')
    okato = models.CharField(
        max_length=11,
        verbose_name=u'ОКАТО')
    oktmo = models.CharField(
        max_length=8,
        verbose_name=u'ОКТМО')
    update_date = models.DateField(
        verbose_name='Дата внесения (обновления) записи',
        blank=False)

    house_number = models.CharField(
        verbose_name=u'Номер дома',
        max_length=20)
    building_number = models.CharField(
        verbose_name=u'Номер корпуса',
        max_length=10)
    structure_number = models.CharField(
        verbose_name=u'Номер строения',
        max_length=10)
    norm_doc = models.ForeignKey(NormativeDocument,
        verbose_name=u'Ссылка на норм. документ',
        null=True,
        blank=True)
    estate_status = models.PositiveSmallIntegerField(
        verbose_name=u'Признак владения',
        blank=False,
        default=0,
        choices=ESTATE_STATUS_CHOICES)

    def address_text(self, levels=None):
        u'''
        Построение полного адреса с учетом вышестоящих АО.
        '''
        # FIXME: Что делать с номерами корпусов и строений?
        parts = []
        if self.parent:
            parts.append(self.parent.address_text(levels=levels))
        parts.append(u'д. {0}'.format(self.house_number))
        return u', '.join(parts)

    def __unicode__(self):
        return self.address_text()

    class Meta:
        verbose_name = u'Запись о доме в ФИАС'
        verbose_name_plural = u'Записи о доме в ФИАС'


class UpdateVersion(models.Model):
    prefix = models.CharField(
        verbose_name=u'Префикс загружаемого файла',
        max_length=20)
    timestamp = models.DateField(
        verbose_name=u'Дата выгрузки файла')
    guid = models.CharField(
        verbose_name=u'Уникальная метка файла выгрузки',
        unique=True,
        max_length=40)

    class Meta:
        verbose_name = u'История обновлений'
        unique_together = ('prefix', 'timestamp')


class HouseInterval(models.Model):
    interval_guid = models.CharField(
        verbose_name=u'Уникальный идентификатор интервала домов',
        max_length=36)
    rec_guid = models.CharField(
        max_length=36,
        verbose_name=u'GUID записи',
        unique=True)
    parent_guid = models.CharField(
        max_length=36,
        verbose_name=u'GUID родительского объекта')
    parent = models.ForeignKey('AddrObject',
        verbose_name=u'Ссылка на родительскую запись',
        null=True)

    postal_code = models.CharField(
        max_length=6,
        verbose_name=u'Почтовый индекс')
    ifnsfl = models.CharField(
        max_length=4,
        verbose_name=u'Код ИФНС ФЛ')
    terr_ifnsfl = models.CharField(
        max_length=4,
        verbose_name=u'Код территориального участка ИФНС ФЛ')
    ifnsul = models.CharField(
        max_length=4,
        verbose_name=u'Код ИФНС ЮЛ')
    terr_ifnsul = models.CharField(
        max_length=4,
        verbose_name=u'Код территориального участка ИФНС ЮЛ')
    okato = models.CharField(
        max_length=11,
        verbose_name=u'ОКАТО')
    oktmo = models.CharField(
        max_length=8,
        verbose_name=u'ОКТМО')
    update_date = models.DateField(
        verbose_name='Дата внесения (обновления) записи',
        blank=False)
    start_date = models.DateField(
        verbose_name='Дата начала действия записи',
        blank=False)
    end_date = models.DateField(
        verbose_name='Дата окончания действия записи',
        blank=False)

    start = models.PositiveIntegerField(
        verbose_name=u'Начало интервала',
        blank=False,
        default=0)
    end = models.PositiveIntegerField(
        verbose_name=u'Окончание интервала',
        blank=False,
        default=0)
    interval_status = models.PositiveIntegerField(
        verbose_name=u'Признак владения',
        blank=False,
        default=0,
        choices=INTERVAL_STATUS_CHOICES)
