#coding: utf-8

from datetime import datetime

from fias.models import UpdateVersion


class UpdateTooOld(Exception):
    pass


class UpdateAlreadyLoaded(Exception):
    pass


def check_update_file(prefix, timestamp, guid):
    # TODO: вставить проверку на корректность данных

    # Каждый файл с XML-данными помечается уникальной меткой, и это поможет
    # нам избежать повторной загрузки
    try:
        UpdateVersion.objects.get(guid=guid)
    except UpdateVersion.DoesNotExist:
        pass
    else:
        raise UpdateAlreadyLoaded('File already loaded: {0}'.format(guid))

    # Если мы уже загрузили более свежую версию, то надо дать отказать в 
    # загрузке устаревшей. Изначальное состояние базы имеет такую же
    # маркировку, как и дельты.
    update_date = datetime.strptime(timestamp, '%Y%m%d')
    try:
        existing = UpdateVersion.objects.get(timestamp__gte=update_date,
                                             prefix=prefix)
    except UpdateVersion.DoesNotExist:
        pass
    else:
        date = existing.timestamp.strftime('%d.%m.%Y (%Y%m%d)')
        raise UpdateTooOld('Newer version of {0} found - {1}'.format(prefix, date))


def register_update_file(prefix, timestamp, guid):
    update_date = datetime.strptime(timestamp, '%Y%m%d')

    UpdateVersion.objects.create(prefix=prefix,
                                 timestamp=update_date,
                                 guid=guid)